/* Placed in public domain by Dmitry Mityugov, 2003 */

#include <nwadv.h>
#include <nwcalls.h>
#include <nwclxcon.h>
#include <nwsm.h>
#include <nwthread.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#define MAX_NLM_NAME_LEN 12

static int is_loaded( NWCONN_HANDLE conn_handle, char const* nlm_name) { /* returns 0 if not loaded, !0 otherwise */
	int result = 0;
	if ( nlm_name != 0 && *nlm_name != 0) {
		size_t nlm_name_len = strlen( nlm_name);
		if ( nlm_name_len <= MAX_NLM_NAME_LEN) {
			NWFSE_NLM_LOADED_LIST nlm_loaded_list;
			NWCCODE ccode;
			nuint32 start_num = 0;
			while (( ccode = NWGetNLMLoadedList( conn_handle, start_num, &nlm_loaded_list)) == 0 && nlm_loaded_list.NLMsInList != 0) {
				size_t i;
				for ( i = 0; i < nlm_loaded_list.NLMsInList; ++i) {
					char filename[ 128]; /* must be larger than MAX_NLM_NAME_LEN */
					char description[ 128];
					char copyright[ 256];
					NWFSE_NLM_INFO nlm_info;
					ccode = NWGetNLMInfo( conn_handle, nlm_loaded_list.NLMNums[ i], filename, description, copyright, &nlm_info);
					if ( ccode == 0) {
						strupr( filename);
						if ( memcmp( filename, nlm_name, nlm_name_len) == 0) {
							if ( filename[ nlm_name_len] == '\0' || filename[ nlm_name_len] == '.') {
								result = !0;
							}
						}
					}
				}
				start_num = nlm_loaded_list.NLMNums[ nlm_loaded_list.NLMsInList - 1] + 1;
			}
		}
	}
	return result;
}
int main( int argc, char* argv[]) {
	char const* server_name;
	char const* user_name;
	char const* password;
	char const* nlm_name;
	unsigned int max_seconds_to_wait;
	NWCCODE ccode;
	NWRCODE rcode;
	NWCONN_HANDLE conn_handle;
	clock_t start_time;
	if ( argc != 5 && argc != 6) {
		SynchronizeStart();
		puts( "\rUsage: RUnload <servername> <username> <password> <nlm_name> [<delay>]");
		return EXIT_FAILURE;
	}
	ccode = NWCallsInit( NULL, NULL);
	if ( ccode != 0) {
		SynchronizeStart();
		printf( "\rRUnload: NWCallsInit failed, ccode = %x\n", ccode);
		return EXIT_FAILURE;
	}
	server_name = strupr( argv[ 1]);
	user_name = strupr( argv[ 2]);
	password = strupr( argv[ 3]);
	nlm_name = strupr( argv[ 4]);
	if ( argc == 6) {
		max_seconds_to_wait = ( unsigned int) atoi( argv[ 5]);
		if ( max_seconds_to_wait > 60 * 60 /* one hour */) {
			SynchronizeStart();
			puts( "\rRUnload: <delay> must be less than or equal to 3600");
			return EXIT_FAILURE;
		}
	}
	else {
		max_seconds_to_wait = 30;
	}
	printf( "\rRUnload: <delay> = %u seconds\n", max_seconds_to_wait);
	rcode = NWCCOpenConnByName( 0,
		server_name,
		NWCC_NAME_FORMAT_BIND,
		NWCC_OPEN_LICENSED,
		NWCC_TRAN_TYPE_WILD,
		&conn_handle);
	if ( rcode != 0) {
		SynchronizeStart();
		printf( "\rRUnload: NWCCOpenConnByName failed, rcode = %x\n", rcode);
		NWCallsTerm( NULL);
		return EXIT_FAILURE;
	}
	ccode = NWLoginToFileServer( conn_handle, user_name, OT_USER, password);
	if ( ccode != 0) {
		SynchronizeStart();
		printf( "\rRUnload: NWLoginToFileServer failed, ccode = %x\n", ccode);
		NWCCCloseConn( conn_handle);
		NWCallsTerm( NULL);
		return EXIT_FAILURE;
	}
	ccode = NWSMUnloadNLM( conn_handle, nlm_name);
	if ( ccode != 0) {
		SynchronizeStart();
		printf( "\rRUnload: NWSMUnloadNLM failed, ccode = %x\n", ccode);
		NWCCCloseConn( conn_handle);
		NWCallsTerm( NULL);
		return EXIT_FAILURE;
	}
	start_time = clock();
	while ( is_loaded( conn_handle, nlm_name) != 0 && clock() - start_time <= max_seconds_to_wait * CLOCKS_PER_SEC) {
		delay( 1000);
	}
	if ( is_loaded( conn_handle, nlm_name) != 0) {
		SynchronizeStart();
		printf( "\rRUnload: Could not unload %s\n", nlm_name);
		NWCCCloseConn( conn_handle);
		NWCallsTerm( NULL);
		return EXIT_FAILURE;
	}
	SynchronizeStart();
	printf( "\rRUnload: %s unloaded\n", nlm_name);
	NWCCCloseConn( conn_handle);
	NWCallsTerm( NULL);
	return EXIT_SUCCESS;
}
